==> spell-0.12.0-py2.py3-none-any.whl: spell_cli/commands/upload.py <==
#!/usr/bin/env python
import os
import threading
import textwrap
from multiprocessing.dummy import Pool

import click
from tus_client.exceptions import TusCommunicationError, TusUploadFailed
from six.moves import queue

from api_client.user_datasets_client import UserDatasetsClient
from api_client.exceptions import ClientException, UnauthorizedRequest
from spell_cli.exceptions import api_client_exception_handler
from spell_cli.log import logger
from spell_cli.utils import prettify_size
from spell_cli.exceptions import ExitException


NUM_WORKERS = 10


@click.command(name="upload",
               short_help="upload a file or directory")
@click.argument("path", type=click.Path(exists=True, dir_okay=True, file_okay=True, readable=True), metavar='PATH')
@click.option("-n", "--name", type=str, help="upload name", default=None, metavar='NAME')
@click.pass_context
def upload(ctx, path, name):
    """
    upload the contents of PATH to uploads/NAME

    The contents of PATH will be uploaded to Spell and will be accessible at
    uploads/NAME. If NAME is not provided and path is a directory, NAME
    defaults to the the directory name of PATH.  If NAME is not provided
    and PATH is a file, the user is prompted for a NAME.
    """
    # get absolute path
    path = os.path.abspath(os.path.expanduser(path))
    # determine upload name
    is_dir = os.path.isdir(path)
    if name is None:
        if is_dir:
            name = os.path.basename(path)
        else:
            name = click.prompt(click.wrap_text("Enter a name for the upload.\n'{}' will be accessible "
                                "at 'uploads/NAME/{}'".format(path, os.path.basename(path))),
                                default=os.path.basename(os.path.dirname(path)), show_default=True)

    resume_command = "spell upload --name '{}' {}".format(name, path)
    resume_message = "To resume upload, run \"{}\"".format(resume_command)

    # create the dataset
    token = ctx.obj["config_handler"].config.token
    ds_client = UserDatasetsClient(token=token, **ctx.obj["client_args"])
    with api_client_exception_handler():
        logger.info("Creating dataset")
        dataset = ds_client.new_dataset(name)
    logger.info("created dataset {}".format(dataset.name))

    # thread pool to perform the uploads
    pool = Pool(NUM_WORKERS)

    # queue to send progress bar updates on
    q = queue.Queue()

    # generate files to upload and add tasks to the executor
    total_bytes = 0
    uploads = []  # a list of arguments for each file to pass to ds_client.upload_file
    if is_dir:
        for (dirpath, dirnames, filenames) in os.walk(path):
            for f in filenames:
                fullpath = os.path.join(dirpath, f)
                relpath = os.path.relpath(fullpath, path)
                if os.name == "nt":
                    relpath = relpath.replace("\\", "/")
                uploads.append((fullpath, dataset.id, relpath, q))
                total_bytes += os.path.getsize(fullpath)
    else:
        uploads.append((path, dataset.id, os.path.basename(path), q))
        total_bytes += os.path.getsize(path)

    click.echo("Total upload size: {}".format(prettify_size(total_bytes)))

    # create thread to update progress
    def update_progress():
        with click.progressbar(length=total_bytes, label="Uploading to uploads/{}".format(name)) as bar:
            while True:
                item = q.get()
                if isinstance(item, int):
                    if int(item) > 0:
                        bar.update(int(item))
                else:
                    break
    t = threading.Thread(target=update_progress)
    t.daemon = True
    t.start()
    try:
        for result in pool.imap_unordered(lambda x: ds_client.upload_file(*x), uploads,
                                          chunksize=max(1, int(len(uploads)/NUM_WORKERS))):
            pass
    except KeyboardInterrupt as e:
        raise ExitException("Upload aborted. {}".format(resume_message))
    except UnauthorizedRequest as e:
        raise ExitException("Login session has expired. Please log in again ('spell login'). {}".format(resume_message))
    except ClientException as e:
        raise ExitException(click.wrap_text("Unexpected upload issue: {}. {}.".format(e.message, resume_message)))
    except TusUploadFailed as e:
        msg = None
        if e.status_code:
            if e.status_code == 423:
                msg = textwrap.dedent("""\
                A file appears to be locked. This could be the result of running 'spell upload'
                on the same set of files. {}""").format(resume_message)
        raise ExitException(msg or click.wrap_text("Unexpected upload issue: {}. {}.".format(e, resume_message)))
    except TusCommunicationError as e:
        msg = None
        if e.status_code:
            if e.status_code == 409:
                msg = textwrap.dedent("""\
                A file appears to have been modified since a previous upload of '{}' was initiated.
                Please restore the upload to its original contents or initiate a new upload under
                a different name. {}.""").format(name, resume_message)
            if e.status_code == 423:
                msg = textwrap.dedent("""\
                A file appears to be locked. This could be the result of running 'spell upload'
                on the same set of files. {}""").format(resume_message)
        raise ExitException(msg or click.wrap_text("Upload issue: {}. {}.".format(e, resume_message)))
    except Exception as e:
        raise ExitException(click.wrap_text("Unexpected upload issue: {}. {}.".format(e, resume_message)))
    finally:
        q.put("exit")
        t.join()
        pool.terminate()

    # complete upload
    ds_client.complete_upload(dataset.id)
    click.echo("Upload of {} ({}) to 'uploads/{}' complete.".format(name, path, name))

==> spell-0.12.3-py2.py3-none-any.whl: spell_cli/commands/upload.py <==
#!/usr/bin/env python
import os
import threading
import textwrap
from multiprocessing.dummy import Pool

import click
from tus_client.exceptions import TusCommunicationError, TusUploadFailed
from six.moves import queue

from api_client.user_datasets_client import UserDatasetsClient
from api_client.exceptions import ClientException, UnauthorizedRequest
from spell_cli.exceptions import api_client_exception_handler
from spell_cli.log import logger
from spell_cli.utils import prettify_size
from spell_cli.exceptions import ExitException


NUM_WORKERS = 10


@click.command(name="upload",
               short_help="Upload a file or directory")
@click.argument("path", type=click.Path(exists=True, dir_okay=True, file_okay=True, readable=True), metavar='PATH')
@click.option("-n", "--name", type=str, help="upload name", default=None, metavar='NAME')
@click.pass_context
def upload(ctx, path, name):
    """
    Upload the contents of PATH to uploads/NAME

    The contents of PATH will be uploaded to Spell and will be accessible at
    uploads/NAME. If NAME is not provided and path is a directory, NAME
    defaults to the the directory name of PATH.  If NAME is not provided
    and PATH is a file, the user is prompted for a NAME.
    """
    # get absolute path
    path = os.path.abspath(os.path.expanduser(path))
    # determine upload name
    is_dir = os.path.isdir(path)
    if name is None:
        if is_dir:
            name = os.path.basename(path)
        else:
            name = click.prompt(click.wrap_text("Enter a name for the upload.\n'{}' will be accessible "
                                "at 'uploads/NAME/{}'".format(path, os.path.basename(path))),
                                default=os.path.basename(os.path.dirname(path)), show_default=True)

    resume_command = "spell upload --name '{}' {}".format(name, path)
    resume_message = "To resume upload, run \"{}\"".format(resume_command)

    # create the dataset
    token = ctx.obj["config_handler"].config.token
    ds_client = UserDatasetsClient(token=token, **ctx.obj["client_args"])
    with api_client_exception_handler():
        logger.info("Creating dataset")
        dataset = ds_client.new_dataset(name)
    logger.info("created dataset {}".format(dataset.name))

    # thread pool to perform the uploads
    pool = Pool(NUM_WORKERS)

    # queue to send progress bar updates on
    q = queue.Queue()

    # generate files to upload and add tasks to the executor
    total_bytes = 0
    uploads = []  # a list of arguments for each file to pass to ds_client.upload_file
    if is_dir:
        for (dirpath, dirnames, filenames) in os.walk(path):
            for f in filenames:
                fullpath = os.path.join(dirpath, f)
                relpath = os.path.relpath(fullpath, path)
                if os.name == "nt":
                    relpath = relpath.replace("\\", "/")
                uploads.append((fullpath, dataset.id, relpath, q))
                total_bytes += os.path.getsize(fullpath)
    else:
        uploads.append((path, dataset.id, os.path.basename(path), q))
        total_bytes += os.path.getsize(path)

    click.echo("Total upload size: {}".format(prettify_size(total_bytes)))

    # create thread to update progress
    def update_progress():
        with click.progressbar(length=total_bytes, label="Uploading to uploads/{}".format(name)) as bar:
            while True:
                item = q.get()
                if isinstance(item, int):
                    if int(item) > 0:
                        bar.update(int(item))
                else:
                    break
    t = threading.Thread(target=update_progress)
    t.daemon = True
    t.start()
    try:
        for result in pool.imap_unordered(lambda x: ds_client.upload_file(*x), uploads,
                                          chunksize=max(1, int(len(uploads)/NUM_WORKERS))):
            pass
    except KeyboardInterrupt as e:
        raise ExitException("Upload aborted. {}".format(resume_message))
    except UnauthorizedRequest as e:
        raise ExitException("Login session has expired. Please log in again ('spell login'). {}".format(resume_message))
    except ClientException as e:
        raise ExitException(click.wrap_text("Unexpected upload issue: {}. {}.".format(e.message, resume_message)))
    except TusUploadFailed as e:
        msg = None
        if e.status_code:
            if e.status_code == 423:
                msg = textwrap.dedent("""\
                A file appears to be locked. This could be the result of running 'spell upload'
                on the same set of files. {}""").format(resume_message)
        raise ExitException(msg or click.wrap_text("Unexpected upload issue: {}. {}.".format(e, resume_message)))
    except TusCommunicationError as e:
        msg = None
        if e.status_code:
            if e.status_code == 409:
                msg = textwrap.dedent("""\
                A file appears to have been modified since a previous upload of '{}' was initiated.
                Please restore the upload to its original contents or initiate a new upload under
                a different name. {}.""").format(name, resume_message)
            if e.status_code == 423:
                msg = textwrap.dedent("""\
                A file appears to be locked. This could be the result of running 'spell upload'
                on the same set of files. {}""").format(resume_message)
        raise ExitException(msg or click.wrap_text("Upload issue: {}. {}.".format(e, resume_message)))
    except Exception as e:
        raise ExitException(click.wrap_text("Unexpected upload issue: {}. {}.".format(e, resume_message)))
    finally:
        q.put("exit")
        t.join()
        pool.terminate()

    # complete upload
    ds_client.complete_upload(dataset.id)
    click.echo("Upload of {} ({}) to 'uploads/{}' complete.".format(name, path, name))
